@@ -242,7 +242,9 @@ hide_popup_window (BonoboUIToolbar *toolbar)
 
 	priv = toolbar->priv;
 
-	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	gdk_display_pointer_ungrab
+		(gtk_widget_get_display (priv->popup_window),
+		 GDK_CURRENT_TIME);
 
 	gtk_grab_remove (priv->popup_window);
 	gtk_widget_hide (priv->popup_window);
@@ -309,11 +311,9 @@ show_popup_window (BonoboUIToolbar *toolbar)
 	BonoboUIToolbarPrivate *priv;
 	const GtkAllocation *toolbar_allocation;
 	gint x, y;
-#ifdef HAVE_GTK_MULTIHEAD
 	GdkScreen *screen;
 	gint screen_width, screen_height;
 	gint window_width, window_height;
-#endif
 
 	priv = toolbar->priv;
 
@@ -328,7 +328,6 @@ show_popup_window (BonoboUIToolbar *toolbar)
 	else
 		y += toolbar_allocation->y + toolbar_allocation->height;
 
-#ifdef HAVE_GTK_MULTIHEAD
 	gtk_window_get_size (GTK_WINDOW (priv->popup_window),
 			     &window_width, &window_height);
 	screen = gtk_widget_get_screen (GTK_WIDGET (toolbar));
@@ -338,7 +337,6 @@ show_popup_window (BonoboUIToolbar *toolbar)
 		x -= window_width;
 	if ((y + window_height) > screen_height)
 		x += toolbar_allocation->width;
-#endif
 
 	gtk_window_move (GTK_WINDOW (priv->popup_window), x, y);
 
@@ -874,6 +872,7 @@ impl_expose_event (GtkWidget *widget,
 {
 	BonoboUIToolbar *toolbar;
 	BonoboUIToolbarPrivate *priv;
+	GtkShadowType shadow_type;
 	GList *p;
 
 	if (! GTK_WIDGET_DRAWABLE (widget))
@@ -882,6 +881,18 @@ impl_expose_event (GtkWidget *widget,
 	toolbar = BONOBO_UI_TOOLBAR (widget);
 	priv = toolbar->priv;
 
+	gtk_widget_style_get (widget, "shadow_type", &shadow_type, NULL);
+
+	gtk_paint_box (widget->style,
+		       widget->window,
+		       GTK_WIDGET_STATE (widget),
+		       shadow_type,
+		       &event->area, widget, "toolbar",
+		       widget->allocation.x,
+		       widget->allocation.y,
+		       widget->allocation.width,
+		       widget->allocation.height);
+
 	for (p = priv->items; p != NULL; p = p->next) {
 		GtkWidget *item_widget;
 
@@ -1093,7 +1104,6 @@ bonobo_ui_toolbar_class_init (BonoboUIToolbarClass *toolbar_class)
 	widget_class->map           = impl_map;
 	widget_class->unmap         = impl_unmap;
 	widget_class->expose_event  = impl_expose_event;
-	widget_class->focus         = bonobo_widget_clobber_focus;
 
 	container_class = GTK_CONTAINER_CLASS (toolbar_class);
 	container_class->remove = impl_remove;
@@ -1159,6 +1169,15 @@ bonobo_ui_toolbar_class_init (BonoboUIToolbarClass *toolbar_class)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE, 0);
+
+	gtk_widget_class_install_style_property (
+	        widget_class,
+		g_param_spec_enum ("shadow_type",
+				   _("Shadow type"),
+				   _("Style of bevel around the toolbar"),
+				   GTK_TYPE_SHADOW_TYPE,
+				   GTK_SHADOW_OUT,
+				   G_PARAM_READABLE));
 }
 
 static void
